@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/auths/spa.c,v 1.10 2009/11/16 19:50:38 nm4 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2003 */
+/* Copyright (c) University of Cambridge 1995 - 2009 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This file, which provides support for Microsoft's Secure Password
@@ -23,8 +25,9 @@ References:
  * typedef unsigned uint32;
  * typedef unsigned char  uint8;
 
-07-August-2003: PH: Patched up the code to avoid assert bombouts for stupid
-                    input data. Find appropriate comment by grepping for "PH".
+07-August-2003:  PH: Patched up the code to avoid assert bombouts for stupid
+                     input data. Find appropriate comment by grepping for "PH".
+16-October-2006: PH: Added a call to auth_check_serv_cond() at the end
 */
 
 
@@ -133,7 +136,7 @@ if (auth_get_no64_data(&data, US"NTLM supported") != OK)
   return FAIL;
   }
 
-if (spa_base64_to_bits((char *)(&request), (const char *)(data)) < 0)
+if (spa_base64_to_bits((char *)(&request), sizeof(request), (const char *)(data)) < 0)
   {
   DEBUG(D_auth) debug_printf("auth_spa_server(): bad base64 data in "
   "request: %s\n", data);
@@ -153,15 +156,13 @@ if (auth_get_no64_data(&data, msgbuf) != OK)
   }
 
 /* dump client response */
-if (spa_base64_to_bits((char *)(&response), (const char *)(data)) < 0)
+if (spa_base64_to_bits((char *)(&response), sizeof(response), (const char *)(data)) < 0)
   {
   DEBUG(D_auth) debug_printf("auth_spa_server(): bad base64 data in "
   "response: %s\n", data);
   return FAIL;
   }
 
-/* get username and put it in $1 */
-
 /***************************************************************
 PH 07-Aug-2003: The original code here was this:
 
@@ -192,10 +193,15 @@ that causes failure if the size of msgbuf is exceeded. ****/
 
 /***************************************************************/
 
-expand_nstring[1] = msgbuf;
+/* Put the username in $auth1 and $1. The former is now the preferred variable;
+the latter is the original variable. */
+
+auth_vars[0] = expand_nstring[1] = msgbuf;
 expand_nlength[1] = Ustrlen(msgbuf);
 expand_nmax = 1;
 
+debug_print_string(ablock->server_debug_string);    /* customized debug */
+
 /* look up password */
 
 clearpass = expand_string(ob->spa_serverpassword);
@@ -226,7 +232,9 @@ if (memcmp(ntRespData,
       ((unsigned char*)responseptr)+IVAL(&responseptr->ntResponse.offset,0),
       24) == 0)
   /* success. we have a winner. */
-  return OK;
+
+  /* Expand server_condition as an authorization check (PH) */
+  return auth_check_serv_cond(ablock);
 
 return FAIL;
 }
@@ -248,27 +256,22 @@ auth_spa_client(
   int buffsize)                          /* size of buffer */
 {
        auth_spa_options_block *ob =
-	       (auth_spa_options_block *)(ablock->options_block);
-       SPAAuthRequest	request;
+               (auth_spa_options_block *)(ablock->options_block);
+       SPAAuthRequest   request;
        SPAAuthChallenge challenge;
-       SPAAuthResponse	response;
+       SPAAuthResponse  response;
        char msgbuf[2048];
        char *domain = NULL;
        char *username, *password;
 
-    if (!smtp_write_command(outblock, FALSE, "AUTH %s\r\n",
-         ablock->public_name))
-	       return FAIL_SEND;
-
-       /* wait for the 3XX OK message */
-       if (!smtp_read_response(inblock, (uschar *)buffer, buffsize, '3', timeout))
-	       return FAIL;
-
        /* Code added by PH to expand the options */
 
+       *buffer = 0;    /* Default no message when cancelled */
+
        username = CS expand_string(ob->spa_username);
        if (username == NULL)
          {
+         if (expand_string_forcedfail) return CANCELLED;
          string_format(buffer, buffsize, "expansion of \"%s\" failed in %s "
            "authenticator: %s", ob->spa_username, ablock->name,
            expand_string_message);
@@ -278,6 +281,7 @@ auth_spa_client(
        password = CS expand_string(ob->spa_password);
        if (password == NULL)
          {
+         if (expand_string_forcedfail) return CANCELLED;
          string_format(buffer, buffsize, "expansion of \"%s\" failed in %s "
            "authenticator: %s", ob->spa_password, ablock->name,
            expand_string_message);
@@ -289,6 +293,7 @@ auth_spa_client(
          domain = CS expand_string(ob->spa_domain);
          if (domain == NULL)
            {
+           if (expand_string_forcedfail) return CANCELLED;
            string_format(buffer, buffsize, "expansion of \"%s\" failed in %s "
              "authenticator: %s", ob->spa_domain, ablock->name,
              expand_string_message);
@@ -298,51 +303,59 @@ auth_spa_client(
 
        /* Original code */
 
+    if (smtp_write_command(outblock, FALSE, "AUTH %s\r\n",
+         ablock->public_name) < 0)
+               return FAIL_SEND;
+
+       /* wait for the 3XX OK message */
+       if (!smtp_read_response(inblock, (uschar *)buffer, buffsize, '3', timeout))
+               return FAIL;
+
        DSPA("\n\n%s authenticator: using domain %s\n\n",
-	       ablock->name, domain);
+               ablock->name, domain);
 
        spa_build_auth_request (&request, CS username, domain);
        spa_bits_to_base64 (US msgbuf, (unsigned char*)&request,
-	       spa_request_length(&request));
+               spa_request_length(&request));
 
        DSPA("\n\n%s authenticator: sending request (%s)\n\n", ablock->name,
-	       msgbuf);
+               msgbuf);
 
        /* send the encrypted password */
-       if (!smtp_write_command(outblock, FALSE, "%s\r\n", msgbuf))
-	       return FAIL_SEND;
+       if (smtp_write_command(outblock, FALSE, "%s\r\n", msgbuf) < 0)
+               return FAIL_SEND;
 
        /* wait for the auth challenge */
        if (!smtp_read_response(inblock, (uschar *)buffer, buffsize, '3', timeout))
-	       return FAIL;
+               return FAIL;
 
        /* convert the challenge into the challenge struct */
        DSPA("\n\n%s authenticator: challenge (%s)\n\n",
-	       ablock->name, buffer + 4);
-       spa_base64_to_bits ((char *)(&challenge), (const char *)(buffer + 4));
+               ablock->name, buffer + 4);
+       spa_base64_to_bits ((char *)(&challenge), sizeof(challenge), (const char *)(buffer + 4));
 
        spa_build_auth_response (&challenge, &response,
-	       CS username, CS password);
+               CS username, CS password);
        spa_bits_to_base64 (US msgbuf, (unsigned char*)&response,
-	       spa_request_length(&response));
+               spa_request_length(&response));
        DSPA("\n\n%s authenticator: challenge response (%s)\n\n", ablock->name,
-	       msgbuf);
+               msgbuf);
 
        /* send the challenge response */
-       if (!smtp_write_command(outblock, FALSE, "%s\r\n", msgbuf))
-	       return FAIL_SEND;
+       if (smtp_write_command(outblock, FALSE, "%s\r\n", msgbuf) < 0)
+               return FAIL_SEND;
 
        /* If we receive a success response from the server, authentication
        has succeeded. There may be more data to send, but is there any point
        in provoking an error here? */
        if (smtp_read_response(inblock, US buffer, buffsize, '2', timeout))
-	       return OK;
+               return OK;
 
        /* Not a success response. If errno != 0 there is some kind of transmission
        error. Otherwise, check the response code in the buffer. If it starts with
        '3', more data is expected. */
        if (errno != 0 || buffer[0] != '3')
-	       return FAIL;
+               return FAIL;
 
        return FAIL;
 }
