@@ -1,10 +1,10 @@
-/* $Cambridge: exim/exim-src/src/local_scan.h,v 1.10 2007/01/08 10:50:18 ph10 Exp $ */
+/* $Cambridge: exim/exim-src/src/local_scan.h,v 1.13 2009/11/16 19:50:37 nm4 Exp $ */
 
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2007 */
+/* Copyright (c) University of Cambridge 1995 - 2009 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This file is the header that is the only Exim header to be included in the
@@ -17,6 +17,7 @@ This API is also used for functions called by the ${dlfunc expansion item. */
 /* Some basic types that make some things easier, the Exim configuration
 settings, and the store functions. */
 
+#include <stdarg.h>
 #include <sys/types.h>
 #include "config.h"
 #include "mytypes.h"
@@ -139,6 +140,8 @@ typedef struct recipient_item {
 
 extern unsigned int debug_selector;    /* Debugging bits */
 
+extern int     body_linecount;         /* Line count in body */
+extern int     body_zerocount;         /* Binary zero count in body */
 extern uschar *expand_string_message;  /* Error info for failing expansion */
 extern uschar *headers_charset;        /* Charset for RFC 2047 decoding */
 extern header_line *header_last;       /* Final header */
@@ -165,7 +168,7 @@ extern int     child_close(pid_t, int);
 extern pid_t   child_open(uschar **, uschar **, int, int *, int *, BOOL);
 extern pid_t   child_open_exim(int *);
 extern pid_t   child_open_exim2(int *, uschar *, uschar *);
-extern void    debug_printf(char *, ...) PRINTF_FUNCTION;
+extern void    debug_printf(char *, ...) PRINTF_FUNCTION(1,2);
 extern uschar *expand_string(uschar *);
 extern void    header_add(int, char *, ...);
 extern void    header_add_at_position(BOOL, uschar *, BOOL, int, char *, ...);
@@ -183,7 +186,8 @@ extern void    receive_add_recipient(uschar *, int);
 extern BOOL    receive_remove_recipient(uschar *);
 extern uschar *rfc2047_decode(uschar *, BOOL, uschar *, int, int *, uschar **);
 extern int     smtp_fflush(void);
-extern void    smtp_printf(char *, ...) PRINTF_FUNCTION;
+extern void    smtp_printf(char *, ...) PRINTF_FUNCTION(1,2);
+extern void    smtp_vprintf(char *, va_list);
 extern uschar *string_copy(uschar *);
 extern uschar *string_copyn(uschar *, int);
 extern uschar *string_sprintf(char *, ...);
