@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/route.c,v 1.14 2009/11/16 19:50:37 nm4 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2003 */
+/* Copyright (c) University of Cambridge 1995 - 2009 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions concerned with routing, and the list of generic router options. */
@@ -32,6 +34,16 @@ optionlist optionlist_routers[] = {
                  (void *)(offsetof(router_instance, address_data)) },
   { "address_test",       opt_bool|opt_public,
                  (void *)(offsetof(router_instance, address_test)) },
+#ifdef EXPERIMENTAL_BRIGHTMAIL
+  { "bmi_deliver_alternate",   opt_bool | opt_public,
+                 (void *)(offsetof(router_instance, bmi_deliver_alternate)) },
+  { "bmi_deliver_default",   opt_bool | opt_public,
+                 (void *)(offsetof(router_instance, bmi_deliver_default)) },
+  { "bmi_dont_deliver",   opt_bool | opt_public,
+                 (void *)(offsetof(router_instance, bmi_dont_deliver)) },
+  { "bmi_rule",           opt_stringptr|opt_public,
+                 (void *)(offsetof(router_instance, bmi_rule)) },
+#endif
   { "cannot_route_message", opt_stringptr | opt_public,
                  (void *)(offsetof(router_instance, cannot_route_message)) },
   { "caseful_local_part", opt_bool | opt_public,
@@ -422,7 +434,8 @@ if (domloc != NULL)
 else
   {
   uschar *address = (sender_address == NULL)? US"" : sender_address;
-  rc = match_address_list(address, TRUE, &list, cache_bits, -1, 0);
+  rc = match_address_list(address, TRUE, TRUE, &list, cache_bits, -1, 0,
+    &sender_data);
   }
 
 switch(rc)
@@ -710,7 +723,7 @@ while ((check = string_nextinlist(&listptr, &sep, buffer, sizeof(buffer)))
     if (pid == 0)
       {
       exim_setugid(uid, gid, TRUE,
-	string_sprintf("require_files check, file=%s", ss));
+        string_sprintf("require_files check, file=%s", ss));
       if (route_check_access(ss, uid, gid, 4)) _exit(0);
       DEBUG(D_route) debug_printf("route_check_access() failed\n");
       _exit(1);
@@ -812,6 +825,7 @@ check_router_conditions(router_instance *r, address_item *addr, int verify,
 {
 int rc;
 uschar *check_local_part;
+unsigned int *localpart_cache;
 
 /* Reset variables to hold a home directory and data from lookup of a domain or
 local part, and ensure search_find_defer is unset, in case there aren't any
@@ -820,6 +834,7 @@ actual lookups. */
 deliver_home = NULL;
 deliver_domain_data = NULL;
 deliver_localpart_data = NULL;
+sender_data = NULL;
 local_user_gid = (gid_t)(-1);
 local_user_uid = (uid_t)(-1);
 search_find_defer = FALSE;
@@ -871,12 +886,17 @@ if ((rc = route_check_dls(r->name, US"domains", r->domains, &domainlist_anchor,
 caseful local part, so that +caseful can restore it, even if this router is
 handling local parts caselessly. However, we can't just pass cc_local_part,
 because that doesn't have the prefix or suffix stripped. A bit of massaging is
-required. */
+required. Also, we only use the match cache for local parts that have not had
+a prefix or suffix stripped. */
 
 if (addr->prefix == NULL && addr->suffix == NULL)
+  {
+  localpart_cache = addr->localpart_cache;
   check_local_part = addr->cc_local_part;
+  }
 else
   {
+  localpart_cache = NULL;
   check_local_part = string_copy(addr->cc_local_part);
   if (addr->prefix != NULL)
     check_local_part += Ustrlen(addr->prefix);
@@ -885,7 +905,7 @@ else
   }
 
 if ((rc = route_check_dls(r->name, US"local_parts", r->local_parts,
-       &localpartlist_anchor, addr->localpart_cache, MCL_LOCALPART,
+       &localpartlist_anchor, localpart_cache, MCL_LOCALPART,
        check_local_part, &deliver_localpart_data, !r->caseful_local_part,
        perror)) != OK)
   return rc;
@@ -936,12 +956,6 @@ if (r->router_home_directory != NULL)
     }
   }
 
-/* This is the point at which we print out the router's debugging string if it
-is set. We wait till here so as to have $home available for local users (and
-anyway, we don't want too much stuff for skipped routers). */
-
-debug_print_string(r->debug_string);
-
 /* Skip if the sender condition is not met. We leave this one till after the
 local user check so that $home is set - enabling the possiblity of letting
 individual recipients specify lists of acceptable/unacceptable senders. */
@@ -950,6 +964,12 @@ if ((rc = route_check_dls(r->name, US"senders", r->senders, NULL,
      sender_address_cache, MCL_ADDRESS, NULL, NULL, FALSE, perror)) != OK)
   return rc;
 
+/* This is the point at which we print out the router's debugging string if it
+is set. We wait till here so as to have $home available for local users (and
+anyway, we don't want too much stuff for skipped routers). */
+
+debug_print_string(r->debug_string);
+
 /* Perform file existence tests. */
 
 if ((rc = check_files(r->require_files, perror)) != OK)
@@ -978,6 +998,46 @@ if (r->condition != NULL)
     }
   }
 
+#ifdef EXPERIMENTAL_BRIGHTMAIL
+/* check if a specific Brightmail AntiSpam rule fired on the message */
+if (r->bmi_rule != NULL) {
+  DEBUG(D_route) debug_printf("checking bmi_rule\n");
+  if (bmi_check_rule(bmi_base64_verdict, r->bmi_rule) == 0) {
+    /* none of the rules fired */
+    DEBUG(D_route)
+      debug_printf("%s router skipped: none of bmi_rule rules fired\n", r->name);
+    return SKIP;
+  };
+};
+
+/* check if message should not be delivered */
+if (r->bmi_dont_deliver) {
+  if (bmi_deliver == 1) {
+    DEBUG(D_route)
+      debug_printf("%s router skipped: bmi_dont_deliver is FALSE\n", r->name);
+    return SKIP;
+  };
+};
+
+/* check if message should go to an alternate location */
+if (r->bmi_deliver_alternate) {
+  if ((bmi_deliver == 0) || (bmi_alt_location == NULL)) {
+    DEBUG(D_route)
+      debug_printf("%s router skipped: bmi_deliver_alternate is FALSE\n", r->name);
+    return SKIP;
+  };
+};
+
+/* check if message should go to default location */
+if (r->bmi_deliver_default) {
+  if ((bmi_deliver == 0) || (bmi_alt_location != NULL)) {
+    DEBUG(D_route)
+      debug_printf("%s router skipped: bmi_deliver_default is FALSE\n", r->name);
+    return SKIP;
+  };
+};
+#endif
+
 /* All the checks passed. */
 
 return OK;
@@ -1023,7 +1083,12 @@ static uschar lastshell[128];
 BOOL
 route_finduser(uschar *s, struct passwd **pw, uid_t *return_uid)
 {
-if (Ustrcmp(lastname, s) != 0)
+BOOL cache_set = (Ustrcmp(lastname, s) == 0);
+
+DEBUG(D_uid) debug_printf("seeking password data for user \"%s\": %s\n", s,
+  cache_set? "using cached result" : "cache not available");
+
+if (!cache_set)
   {
   int i = 0;
 
@@ -1050,6 +1115,7 @@ if (Ustrcmp(lastname, s) != 0)
 
   else for (;;)
     {
+    errno = 0;
     if ((lastpw = getpwnam(CS s)) != NULL) break;
     if (++i > finduser_retries) break;
     sleep(1);
@@ -1068,14 +1134,25 @@ if (Ustrcmp(lastname, s) != 0)
     pwcopy.pw_shell = CS lastshell;
     lastpw = &pwcopy;
     }
+
+  else DEBUG(D_uid)
+    {
+    if (errno != 0) debug_printf("getpwnam(%s) failed: %s\n", s,
+      strerror(errno));
+    }
+  }
+
+if (lastpw == NULL)
+  {
+  DEBUG(D_uid) debug_printf("getpwnam() returned NULL (user not found)\n");
+  return FALSE;
   }
 else
   {
-  DEBUG(D_uid) debug_printf("finduser used cached passwd data for %s\n", s);
+  DEBUG(D_uid) debug_printf("getpwnam() succeeded uid=%d gid=%d\n",
+    lastpw->pw_uid, lastpw->pw_gid);
   }
 
-if (lastpw == NULL) return FALSE;
-
 if (return_uid != NULL) *return_uid = lastpw->pw_uid;
 if (pw != NULL) *pw = lastpw;
 
@@ -1643,8 +1720,8 @@ for (r = (addr->start_router == NULL)? routers : addr->start_router;
 
   HDEBUG(D_route) debug_printf("calling %s router\n", r->name);
 
-  yield = (r->info->code)(r, addr, pw, verify != v_none, paddr_local,
-    paddr_remote, addr_new, addr_succeed);
+  yield = (r->info->code)(r, addr, pw, verify, paddr_local, paddr_remote,
+    addr_new, addr_succeed);
 
   if (yield == FAIL)
     {
@@ -1800,7 +1877,7 @@ if (r->translate_ip_address != NULL)
 
     DEBUG(D_route) debug_printf("%s [%s] translated to %s\n",
       h->name, h->address, newaddress);
-    if (string_is_ip_address(newaddress, NULL))
+    if (string_is_ip_address(newaddress, NULL) != 0)
       {
       h->address = newaddress;
       continue;
@@ -1813,7 +1890,7 @@ if (r->translate_ip_address != NULL)
     h->mx = MX_NONE;
 
     store_pool = POOL_PERM;
-    rc = host_find_byname(h, NULL, NULL, TRUE);
+    rc = host_find_byname(h, NULL, HOST_FIND_QUALIFY_SINGLE, NULL, TRUE);
     store_pool = old_pool;
 
     if (rc == HOST_FIND_FAILED || rc == HOST_FIND_AGAIN)
@@ -1835,8 +1912,6 @@ yield = exp_bool(addr, r->name, US"unseen", r->unseen, r->expand_unseen,
   &unseen);
 if (yield != OK) goto ROUTE_EXIT;
 
-
-
 /* Debugging output recording a successful routing */
 
 HDEBUG(D_route)
@@ -1858,10 +1933,11 @@ DEBUG(D_route)
 
   for (h = addr->host_list; h != NULL; h = h->next)
     {
-    debug_printf("  host %s ", h->name);
-    if (h->address != NULL) debug_printf("[%s] ", h->address);
-    if (h->mx >= 0) debug_printf("MX=%d", h->mx);
-      else if (h->mx != MX_NONE) debug_printf("rgroup=%d", h->mx);
+    debug_printf("  host %s", h->name);
+    if (h->address != NULL) debug_printf(" [%s]", h->address);
+    if (h->mx >= 0) debug_printf(" MX=%d", h->mx);
+      else if (h->mx != MX_NONE) debug_printf(" rgroup=%d", h->mx);
+    if (h->port != PORT_NONE) debug_printf(" port=%d", h->port);
     debug_printf("\n");
     }
   }
