@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/smtp_out.c,v 1.10 2009/11/16 19:50:37 nm4 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2003 */
+/* Copyright (c) University of Cambridge 1995 - 2009 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* A number of functions for driving outgoing SMTP calls. */
@@ -107,7 +109,7 @@ if (pstring == NULL)
   {
   addr->transport_return = PANIC;
   addr->message = string_sprintf("failed to expand \"%s\" (\"port\" option) "
-    "for %s: %s", msg, expand_string_message);
+    "for %s: %s", rstring, msg, expand_string_message);
   return FALSE;
   }
 
@@ -152,10 +154,15 @@ detected by checking for a colon in the address. AF_INET6 is defined even on
 non-IPv6 systems, to enable the code to be less messy. However, on such systems
 host->address will always be an IPv4 address.
 
+The port field in the host item is used if it is set (usually router from SRV
+records or elsewhere). In other cases, the default passed as an argument is
+used, and the host item is updated with its value.
+
 Arguments:
-  host        host item containing name and address
+  host        host item containing name and address (and sometimes port)
   host_af     AF_INET or AF_INET6
-  port        remote port to connect to, in host byte order
+  port        default remote port to connect to, in host byte order, for those
+                hosts whose port setting is PORT_NONE
   interface   outgoing interface address or NULL
   timeout     timeout value or 0
   keepalive   TRUE to use keepalive
@@ -168,8 +175,18 @@ smtp_connect(host_item *host, int host_af, int port, uschar *interface,
   int timeout, BOOL keepalive)
 {
 int on = 1;
+int save_errno = 0;
 int sock;
 
+if (host->port != PORT_NONE)
+  {
+  HDEBUG(D_transport|D_acl|D_v)
+    debug_printf("Transport port=%d replaced by host-specific port=%d\n", port,
+      host->port);
+  port = host->port;
+  }
+else host->port = port;    /* Set the port actually used */
+
 HDEBUG(D_transport|D_acl|D_v)
   {
   if (interface == NULL)
@@ -191,23 +208,51 @@ setsockopt(sock, IPPROTO_TCP, TCP_NODELAY, (uschar *)(&on), sizeof(on));
 is the same type (IPv4 or IPv6) as the outgoing address. */
 
 if (interface != NULL && ip_bind(sock, host_af, interface, 0) < 0)
-  log_write(0, LOG_MAIN|LOG_PANIC_DIE, "unable to bind outgoing SMTP call "
-    "to %s: %s", interface, strerror(errno));
+  {
+  save_errno = errno;
+  HDEBUG(D_transport|D_acl|D_v)
+    debug_printf("unable to bind outgoing SMTP call to %s: %s", interface,
+    strerror(errno));
+  }
 
 /* Connect to the remote host, and add keepalive to the socket before returning
 it, if requested. */
 
-if (ip_connect(sock, host_af, host->address, port, timeout) < 0)
+else if (ip_connect(sock, host_af, host->address, port, timeout) < 0)
+  save_errno = errno;
+
+/* Either bind() or connect() failed */
+
+if (save_errno != 0)
   {
-  int save_errno = errno;
-  HDEBUG(D_transport|D_acl|D_v) debug_printf("failed\n");
-  close(sock);
+  HDEBUG(D_transport|D_acl|D_v)
+    {
+    debug_printf("failed: %s", CUstrerror(save_errno));
+    if (save_errno == ETIMEDOUT)
+      debug_printf(" (timeout=%s)", readconf_printtime(timeout));
+    debug_printf("\n");
+    }
+  (void)close(sock);
   errno = save_errno;
   return -1;
   }
+
+/* Both bind() and connect() succeeded */
+
 else
   {
+  union sockaddr_46 interface_sock;
+  EXIM_SOCKLEN_T size = sizeof(interface_sock);
   HDEBUG(D_transport|D_acl|D_v) debug_printf("connected\n");
+  if (getsockname(sock, (struct sockaddr *)(&interface_sock), &size) == 0)
+    sending_ip_address = host_ntoa(-1, &interface_sock, NULL, &sending_port);
+  else
+    {
+    log_write(0, LOG_MAIN | ((errno == ECONNRESET)? 0 : LOG_PANIC),
+      "getsockname() failed: %s", strerror(errno));
+    close(sock);
+    return -1;
+    }
   if (keepalive) ip_keepalive(sock, host->address, TRUE);
   return sock;
   }
